# print('Executing expLogbook.py')
# from getComment4File import *
import requests
import urllib.parse
import os
import builtins
import re
from elog.logbook_exceptions import *
from datetime import datetime
from time import localtime

# disable warnings about ssl verification
from requests.packages.urllib3.exceptions import InsecureRequestWarning
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)


class myLogbook(object):
    """
    Logbook provides methods to interface with logbook on location: "server:port/subdir/logbook". User can create,
    edit, delete logbook messages.
    """

    def __init__(self, hostname, logbook='', port=None, user=None, password=None, subdir='', use_ssl=True,
                 encrypt_pwd=True):
        """
        :param hostname: elog server hostname. If whole url is specified here, it will be parsed and arguments:
                         "logbook, port, subdir, use_ssl" will be overwritten by parsed values.
        :param logbook: name of the logbook on the elog server
        :param port: elog server port (if not specified will default to '80' if use_ssl=False or '443' if use_ssl=True
        :param user: username (if authentication needed)
        :param password: password (if authentication needed) Password will be encrypted with sha256 unless
                         encrypt_pwd=False (default: True)
        :param subdir: subdirectory of logbooks locations
        :param use_ssl: connect using ssl (ignored if url starts with 'http://'' or 'https://'?
        :param encrypt_pwd: To avoid exposing password in the code, this flag can be set to False and password
                            will then be handled as it is (user needs to provide sha256 encrypted password with
                            salt= '' and rounds=5000)
        :return:
        """
        hostname = hostname.strip()
        
        # parse url to see if some parameters are defined with url
        parsed_url = urllib.parse.urlsplit(hostname)

        # ---- handle SSL -----
        # hostname must be modified according to use_ssl flag. If hostname starts with https:// or http://
        # the use_ssl flag is ignored
        url_scheme = parsed_url.scheme
        if url_scheme == 'http':
            use_ssl = False

        elif url_scheme == 'https':
            use_ssl = True

        elif not url_scheme:
            # add http or https
            if use_ssl:
                url_scheme = 'https'
            else:
                url_scheme = 'http'

        # ---- handle port -----
        # 1) by default use port defined in the url
        # 2) remove any 'default' ports such as 80 for http and 443 for https
        # 3) if port not defined in url and not 'default' add it to netloc
        
        netloc = parsed_url.netloc
        if netloc == "" and "localhost" in hostname:
            netloc = 'localhost'
        netloc_split = netloc.split(':')
        if len(netloc_split) > 1:
            # port defined in url --> remove if needed
            port = netloc_split[1]
            if (port == 80 and not use_ssl) or (port == 443 and use_ssl):
                netloc = netloc_split[0]

        else:
            # add port info if needed
            if port is not None and not (port == 80 and not use_ssl) and not (port == 443 and use_ssl):
                netloc += ':{}'.format(port)

        # ---- handle subdir and logbook -----
        # parsed_url.path = /<subdir>/<logbook>/

        # Remove last '/' for easier parsing
        url_path = parsed_url.path
        if url_path.endswith('/'):
            url_path = url_path[:-1]

        splitted_path = url_path.split('/')
        if url_path and len(splitted_path) > 1:
            # If here ... then at least some part of path is defined.

            # If logbook defined --> treat path current path as subdir and add logbook at the end
            # to define the full path. Else treat existing path as <subdir>/<logbook>.
            # Put first and last '/' back on its place
            if logbook:
                url_path += '/{}'.format(logbook)
            else:
                logbook = splitted_path[-1]
            
        else:
            # There is nothing. Use arguments.
            url_path = subdir + '/' + logbook

        # urllib.parse.quote replaces special characters with %xx escapes
        # self._logbook_path = urllib.parse.quote('/' + url_path + '/').replace('//', '/')
        self._logbook_path = ('/' + url_path + '/').replace('//', '/')
        
        self._url = url_scheme + '://' + netloc + self._logbook_path
        self.logbook = logbook
        self._user = user
        self._password = _handle_pswd(password, encrypt_pwd)

    def post(self, message, msg_id=None, reply=False, attributes=None, attachments=None, encoding=None,
             **kwargs):
        """
        Posts message to the logbook. If msg_id is not specified new message will be created, otherwise existing
        message will be edited, or a reply (if reply=True) to it will be created. This method returns the msg_id
        of the newly created message.

        :param message: string with message text
        :param msg_id: ID number of message to edit or reply. If not specified new message is created.
        :param reply: If 'True' reply to existing message is created instead of editing it
        :param attributes: Dictionary of attributes. Following attributes are used internally by the elog and will be
                           ignored: Text, Date, Encoding, Reply to, In reply to, Locked by, Attachment
        :param attachments: list of:
                                  - file like objects which read() will return bytes (if file_like_object.name is not
                                    defined, default name "attachment<i>" will be used.
                                  - paths to the files
                            All items will be appended as attachment to the elog entry. In case of unknown
                            attachment an exception LogbookInvalidAttachment will be raised.
        :param encoding: Defines encoding of the message. Can be: 'plain' -> plain text, 'html'->html-text,
                         'ELCode' --> elog formatting syntax
        :param kwargs: Anything in the kwargs will be interpreted as attribute. e.g.: logbook.post('Test text',
                       Author='Rok Vintar), "Author" will be sent as an attribute. If named same as one of the
                       attributes defined in "attributes", kwargs will have priority.

        :return: msg_id
        """

        attributes = attributes or {}
        attributes = {**attributes, **kwargs}  # kwargs as attributes with higher priority

        attachments = attachments or []

        if encoding is not None:
            if encoding not in ['plain', 'HTML', 'ELCode']:
                raise LogbookMessageRejected('Invalid message encoding. Valid options: plain, HTML, ELCode.')
            attributes['Encoding'] = encoding

        attributes_to_edit = dict()
        if msg_id:
            # Message exists, we can continue
            if reply:
                # Verify that there is a message on the server, otherwise do not reply to it!
                self._check_if_message_on_server(msg_id)  # raises exception in case of none existing message

                attributes['reply_to'] = str(msg_id)

            else:  # Edit existing
                attributes['edit_id'] = str(msg_id)
                attributes['skiplock'] = '1'

                # Handle existing attachments
                msg_to_edit, attributes_to_edit, attach_to_edit = self.read(msg_id)

                i = 0
                for attachment in attach_to_edit:
                    if attachment:
                        # Existing attachments must be passed as regular arguments attachment<i> with value= file name
                        # Read message returnes full urls to existing attachments:
                        # <hostname>:[<port>][/<subdir]/<logbook>/<msg_id>/<file_name>
                        attributes['attachment' + str(i)] = os.path.basename(attachment)
                        i += 1

                for attribute, data in attributes.items():
                    new_data = attributes.get(attribute)
                    if new_data is not None:
                        attributes_to_edit[attribute] = new_data
        else:
            # As we create a new message, specify creation time if not already specified in attributes
            if 'When' not in attributes:
                attributes['When'] = int(datetime.now().timestamp())

        if not attributes_to_edit:
            attributes_to_edit = attributes
        # Remove any attributes that should not be sent
        _remove_reserved_attributes(attributes_to_edit)

        if attachments:
            files_to_attach, objects_to_close = self._prepare_attachments(attachments)
        else:
            objects_to_close = list()
            files_to_attach = list()

        # Make requests module think that Text is a "file". This is the only way to force requests to send data as
        # multipart/form-data even if there are no attachments. Elog understands only multipart/form-data
        files_to_attach.append(('Text', ('', message)))

        # Base attributes are common to all messages
        self._add_base_msg_attributes(attributes_to_edit)
        
        # Keys in attributes cannot have certain characters like whitespaces or dashes for the http request
        attributes_to_edit = _replace_special_characters_in_attribute_keys(attributes_to_edit)

        try:
            response = requests.post(self._url, data=attributes_to_edit, files=files_to_attach, allow_redirects=False,
                                     verify=False)
            # Validate response. Any problems will raise an Exception.
            resp_message, resp_headers, resp_msg_id = _validate_response(response)

            # Close file like objects that were opened by the elog (if  path
            for file_like_object in objects_to_close:
                if hasattr(file_like_object, 'close'):
                    file_like_object.close()

        except requests.RequestException as e:
            # Check if message on server.
            self._check_if_message_on_server(msg_id)  # raises exceptions if no message or no response from server

            # If here: message is on server but cannot be downloaded (should never happen)
            raise LogbookServerProblem('Cannot access logbook server to post a message, ' + 'because of:\n' +
                                       '{0}'.format(e))

        # Any error before here should raise an exception, but check again for nay case.
        if not resp_msg_id or resp_msg_id < 1:
            raise LogbookInvalidMessageID('Invalid message ID: ' + str(resp_msg_id) + ' returned')
        return resp_msg_id

    def read(self, msg_id):
        """
        Reads message from the logbook server and returns tuple of (message, attributes, attachments) where:
        message: string with message body
        attributes: dictionary of all attributes returned by the logbook
        attachments: list of urls to attachments on the logbook server

        :param msg_id: ID of the message to be read
        :return: message, attributes, attachments
        """

        request_headers = dict()
        if self._user or self._password:
            request_headers['Cookie'] = self._make_user_and_pswd_cookie()

        try:
            self._check_if_message_on_server(msg_id)  # raises exceptions if no message or no response from server
            response = requests.get(self._url + str(msg_id) + '?cmd=download', headers=request_headers,
                                    allow_redirects=False, verify=False)

            # Validate response. If problems Exception will be thrown.
            resp_message, resp_headers, resp_msg_id = _validate_response(response)

        except requests.RequestException as e:
            # If here: message is on server but cannot be downloaded (should never happen)
            raise LogbookServerProblem('Cannot access logbook server to read the message with ID: ' + str(msg_id) +
                                       'because of:\n' + '{0}'.format(e))

        # Parse message to separate message body, attributes and attachments
        attributes = dict()
        attachments = list()

        returned_msg = resp_message.decode('utf-8', 'ignore').splitlines()
        delimiter_idx = returned_msg.index('========================================')

        message = '\n'.join(returned_msg[delimiter_idx + 1:])
        for line in returned_msg[0:delimiter_idx]:
            line = line.split(': ')
            data = ''.join(line[1:])
            if line[0] == 'Attachment':
                attachments = data.split(',')
                # Here are only attachment names, make a full url out of it, so they could be
                # recognisable by others, and downloaded if needed
                attachments = [self._url + '{0}'.format(i) for i in attachments]
            else:
                attributes[line[0]] = data

        return message, attributes, attachments

    def delete(self, msg_id):
        """
        Deletes message thread (!!!message + all replies!!!) from logbook.
        It also deletes all of attachments of corresponding messages from the server.

        :param msg_id: message to be deleted
        :return:
        """

        request_headers = dict()
        if self._user or self._password:
            request_headers['Cookie'] = self._make_user_and_pswd_cookie()

        try:
            self._check_if_message_on_server(msg_id)  # check if something to delete

            response = requests.get(self._url + str(msg_id) + '?cmd=Delete&confirm=Yes', headers=request_headers,
                                    allow_redirects=False, verify=False)

            _validate_response(response)  # raises exception if any other error identified

        except requests.RequestException as e:
            # If here: message is on server but cannot be downloaded (should never happen)
            raise LogbookServerProblem('Cannot access logbook server to delete the message with ID: ' + str(msg_id) +
                                       'because of:\n' + '{0}'.format(e))

        # Additional validation: If successfully deleted then status_code = 302. In case command was not executed at
        # all (not English language --> no download command supported) status_code = 200 and the content is just a
        # html page of this whole message.
        if response.status_code == 200:
            raise LogbookServerProblem('Cannot process delete command (only logbooks in English supported).')

    def search(self, search_term, n_results = 20, scope="subtext"):
        """
        Searches the logbook and returns the message ids.

        """
        request_headers = dict()
        if self._user or self._password:
            request_headers['Cookie'] = self._make_user_and_pswd_cookie()

        # Putting n_results = 0 crashes the elog. also in the web-gui.
        n_results = 1 if n_results < 1 else n_results

        params = {
            "mode": "full",
            "reverse": "1",
            "npp": n_results
        }
        if type(search_term) is dict:
            params.update(search_term)
        else:
            params.update({scope : search_term})        

        try:
            response = requests.get(self._url, params=params, headers=request_headers,
                                    allow_redirects=False, verify=False)

            # Validate response. If problems Exception will be thrown.
            _validate_response(response)
            resp_message = response

        except requests.RequestException as e:
            # If here: message is on server but cannot be downloaded (should never happen)
            raise LogbookServerProblem('Cannot access logbook server to read message ids '
                                       'because of:\n' + '{0}'.format(e))

        from lxml import html
        tree = html.fromstring(resp_message.content)
        message_ids = tree.xpath('(//tr/td[@class="list1" or @class="list2"][1])/a/@href')
        message_ids = [int(m.split("/")[-1]) for m in message_ids]
        return message_ids


    def get_last_message_id(self):
        ids = self.get_message_ids()
        if len(ids) > 0:
            return ids[0]
        else:
            return None

    def get_message_ids(self):
        request_headers = dict()
        if self._user or self._password:
            request_headers['Cookie'] = self._make_user_and_pswd_cookie()

        try:
            response = requests.get(self._url + 'page', headers=request_headers,
                                    allow_redirects=False, verify=False)

            # Validate response. If problems Exception will be thrown.
            _validate_response(response)
            resp_message = response

        except requests.RequestException as e:
            # If here: message is on server but cannot be downloaded (should never happen)
            raise LogbookServerProblem('Cannot access logbook server to read message ids '
                                       'because of:\n' + '{0}'.format(e))

        from lxml import html
        tree = html.fromstring(resp_message.content)
        message_ids = tree.xpath('(//tr/td[@class="list1" or @class="list2"][1])/a/@href')
        message_ids = [int(m.split("/")[-1]) for m in message_ids]
        return message_ids

    def _check_if_message_on_server(self, msg_id):
        """Try to load page for specific message. If there is a htm tag like <td class="errormsg"> then there is no
        such message.

        :param msg_id: ID of message to be checked
        :return:
        """

        request_headers = dict()
        if self._user or self._password:
            request_headers['Cookie'] = self._make_user_and_pswd_cookie()
        try:
            response = requests.get(self._url + str(msg_id), headers=request_headers, allow_redirects=False,
                                    verify=False)

            # If there is no message code 200 will be returned (OK) and _validate_response will not recognise it
            # but there will be some error in the html code.
            resp_message, resp_headers, resp_msg_id = _validate_response(response)
            # If there is no message, code 200 will be returned (OK) but there will be some error indication in
            # the html code.
            if re.findall('<td.*?class="errormsg".*?>.*?</td>',
                          resp_message.decode('utf-8', 'ignore'),
                          flags=re.DOTALL):
                raise LogbookInvalidMessageID('Message with ID: ' + str(msg_id) + ' does not exist on logbook.')

        except requests.RequestException as e:
            raise LogbookServerProblem('No response from the logbook server.\nDetails: ' + '{0}'.format(e))

    def _add_base_msg_attributes(self, data):
        """
        Adds base message attributes which are used by all messages.
        :param data: dict of current attributes
        :return: content string
        """
        data['cmd'] = 'Submit'
        data['exp'] = self.logbook
        if self._user:
            data['unm'] = self._user
        if self._password:
            data['upwd'] = self._password

    def _prepare_attachments(self, files):
        """
        Parses attachments to content objects. Attachments can be:
            - file like objects: must have method read() which returns bytes. If it has attribute .name it will be used
              for attachment name, otherwise generic attribute<i> name will be used.
            - path to the file on disk

        Note that if attachment is is an url pointing to the existing Logbook server it will be ignored and no
        exceptions will be raised. This can happen if attachments returned with read_method are resend.

        :param files: list of file like objects or paths
        :return: content string
        """
        prepared = list()
        i = 0
        objects_to_close = list()  # objects that are created (opened) by elog must be later closed
        for file_obj in files:
            if hasattr(file_obj, 'read'):
                i += 1
                attribute_name = 'attfile' + str(i)

                filename = attribute_name  # If file like object has no name specified use this one
                candidate_filename = os.path.basename(file_obj.name)

                if filename:  # use only if not empty string
                    filename = candidate_filename

            elif isinstance(file_obj, str):
                # Check if it is:
                #           - a path to the file --> open file and append
                #           - an url pointing to the existing Logbook server --> ignore

                filename = ""
                attribute_name = ""

                if os.path.isfile(file_obj):
                    i += 1
                    attribute_name = 'attfile' + str(i)

                    file_obj = builtins.open(file_obj, 'rb')
                    filename = os.path.basename(file_obj.name)

                    objects_to_close.append(file_obj)

                elif not file_obj.startswith(self._url):
                    raise LogbookInvalidAttachmentType('Invalid type of attachment: \"' + file_obj + '\".')
            else:
                raise LogbookInvalidAttachmentType('Invalid type of attachment[' + str(i) + '].')

            prepared.append((attribute_name, (filename, file_obj)))

        return prepared, objects_to_close

    def _make_user_and_pswd_cookie(self):
        """
        prepares user name and password cookie. It is sent in header when posting a message.
        :return: user name and password value for the Cookie header
        """
        cookie = ''
        if self._user:
            cookie += 'unm=' + self._user + ';'
        if self._password:
            cookie += 'upwd=' + self._password + ';'

        return cookie

    def getMessage4File(self, filename):
        ids = self.search({'file':filename})
        if len(ids) > 0:
            message, _ , _ = self.read(ids[0])
            return message
        else:
            return None

    def getShortMessage4File(self,filename):
        stuff = self.getMessage4File(filename)
        import html2text
        h = html2text.HTML2Text()
        # Ignore converting links from HTML
        h.ignore_links = True
        return h.handle(stuff)

def _remove_reserved_attributes(attributes):
    """
    Removes elog reserved attributes (from the attributes dict) that can not be sent.

    :param attributes: dictionary of attributes to be cleaned.
    :return:
    """

    if attributes:
        attributes.get('$@MID@$', None)
        attributes.pop('Date', None)
        attributes.pop('Attachment', None)
        attributes.pop('Text', None)  # Remove this one because it will be send attachment like


def _replace_special_characters_in_attribute_keys(attributes):
    """
    Replaces special characters in elog attribute keys by underscore, otherwise attribute values will be erased in
    the http request. This is using the same replacement elog itself is using to handle these cases

    :param attributes: dictionary of attributes to be cleaned.
    :return: attributes with replaced keys
    """
    return {re.sub('[^0-9a-zA-Z]', '_', key): value for key, value in attributes.items()}


def _validate_response(response):
    """ Validate response of the request."""

    msg_id = None

    if response.status_code not in [200, 302]:
        # 200 --> OK; 302 --> Found
        # Html page is returned with error description (handling errors same way as on original client. Looks
        # like there is no other way.

        err = re.findall('<td.*?class="errormsg".*?>.*?</td>',
                         response.content.decode('utf-8', 'ignore'),
                         flags=re.DOTALL)

        if len(err) > 0:
            # Remove html tags
            # If part of the message has: Please go  back... remove this part since it is an instruction for
            # the user when using browser.
            err = re.sub('(?:<.*?>)', '', err[0])
            if err:
                raise LogbookMessageRejected('Rejected because of: ' + err)
            else:
                raise LogbookMessageRejected('Rejected because of unknown error.')

        # Other unknown errors
        raise LogbookMessageRejected('Rejected because of unknown error.')
    else:
        location = response.headers.get('Location')
        if location is not None:
            if 'has moved' in location:
                raise LogbookServerProblem('Logbook server has moved to another location.')
            elif 'fail' in location:
                raise LogbookAuthenticationError('Invalid username or password.')
            else:
                # returned locations is something like: '<host>/<sub_dir>/<logbook>/<msg_id><query>
                # with urllib.parse.urlparse returns attribute path=<sub_dir>/<logbook>/<msg_id>
                msg_id = int(urllib.parse.urlsplit(location).path.split('/')[-1])

        if b'form name=form1' in response.content or b'type=password' in response.content:
            # Not to smart to check this way, but no other indication of this kind of error.
            # C client does it the same way
            raise LogbookAuthenticationError('Invalid username or password.')

    return response.content, response.headers, msg_id


def _handle_pswd(password, encrypt=True):
    """
    Takes password string and returns password as needed by elog. If encrypt=True then password will be
    sha256 encrypted (salt='', rounds=5000). Before returning password, any trailing $5$$ will be removed
    independent off encrypt flag.

    :param password: password string
    :param encrypt: encrypt password?
    :return: elog prepared password
    """
    if encrypt and password:
        from passlib.hash import sha256_crypt
        return sha256_crypt.encrypt(password, salt='', rounds=5000)[4:]
    elif password and password.startswith('$5$$'):
        return password[4:]
    else:
        return password


##########################################################################################################
# ogf settings:
try:
    from libhreels.eLogCredentials import user, password, dummy, unsafe    # Defines User credentials
    logbook = myLogbook('https://labor-ep3.physik.uni-halle.de/HREELS/', user=dummy, password=unsafe)
    print('eLog available!')
    available = True
except:
    available = False




#!/usr/bin/python
# LEED library Version 0.1    (July 2019 @ WFW)
#
import numpy as np
import re
import matplotlib.pyplot as plt
from scipy import optimize, interpolate
from datetime import datetime
from datetime import timedelta
from matplotlib.transforms import offset_copy
# from pylab import figure
from matplotlib.pyplot import figure
import cv2
import os

def offset(ax, x, y):
    return offset_copy(ax.transData, x=x, y=y, units='dots')

def LEED_elog_Dictionary(file, path):
    spec = LEED(file,datapath=path)
    dic = {'Category':'LEED', 'Type':'LEED', 'Substrate':'as before', 
        'Summary':spec.energy+' eV', 
        'Temperature':'', 'File':str(file), 
        'Scan_Range':'', 'Energy':spec.energy,
        'N_channels':'', 'MDate':spec.startTimeEpoch, 
        'Author':'Python Script'}
    return dic

class LEED:
    """This class handles LEED data and their plotting."""
    path = "./"
    def __init__(self, _filename, datapath=path):
        self.datapath = datapath
        self.readDataFile(_filename)

    def readDataFile(self, _filename):
        # if _filename.find(".") == -1:
        #     _filename +=".aes"
        self.filename = _filename
        self.fname = self.filename[0:self.filename.find(".")]
        import glob
        f = glob.glob(self.datapath+"**/"+self.filename, recursive=True)
        if len(f) == 0:
            print('Error: No files found!')
            print('Not found:',self.filename)
            return
        f = glob.glob(self.datapath+"**/"+self.filename, recursive=True)[0]
        self.datapath = os.path.dirname(f)
        self.startTime = datetime.fromtimestamp(os.path.getmtime(f))        # Only the file saving time
        self.startTimeEpoch = os.path.getmtime(f)
        match = re.search(r"\_(\d*)eV",self.fname)
        self.energy = match.group(1)
        self.picture = cv2.imread(f)

    def __str__(self):
        return self.datapath+self.filename

    def info(self):
        print("filename: \t\t",self.filename)
        print("datapath: \t\t",self.datapath)
        print("startTime: \t\t",self.startTime)
        print("energy: \t\t",self.energy)
        print('--------')

    def plot(self):
        new_pic = self.picture
        # Pmin = min(new_pic.all())
        # Pmax = max(new_pic.any())
        # new_pic = (new_pic-Pmin)/(Pmax-Pmin)*255
        cv2.imshow('LEED'+self.fname,new_pic)
        cv2.waitKey(0)
        cv2.destroyAllWindows()

if __name__ == '__main__':
    path = 'D:\Data\Python\LEED'
    d1 = LEED("190122_L_01_42eV.tif",datapath=path)
    
    d1.info()
    d1.plot()
    
    import code
    code.interact(local=locals())



# Testing dielectric models and their surface loss function
import matplotlib.pyplot as plt
import numpy as np
from numpy import real, imag, sqrt
#from scipy.constants import physical_constants

# Thz = 100*physical_constants['speed of light in vacuum'][0] # conversion from cm^-1 to Hz 

def doping2plasmaFrequency(doping,epsInfinity=1.):
    '''Returns the bulk plasma frequency in cm-1 for a doping 
       given by the argument in cm-3.
       Check if eps_Infinity handling is correct.'''
    return np.sqrt(doping)*1.8817885780819758e-06/np.sqrt(epsInfinity)

def doping2surfacePlasma(doping,epsInfinity=1.):
    '''Returns the surface plasma frequency in cm-1 for a doping 
       given by the argument in cm-3'''
    return np.sqrt(doping)*1.8817885780819758e-06/np.sqrt(1+epsInfinity)

def loss(eps):
    return -np.imag(eps)

def surfaceLoss(eps):
    return np.imag(1/(1+eps))

def reflectivity(eps):  # IR reflectivity
    sq = np.sqrt(eps)
    a =np.abs((sq-1)/(sq+1))
    return a*a

def sigma(eps,w):       # Complex conductivity (ignored factor eps_0)
    return (eps-1)*w/1j

def plotDielectrics(x,eps):
    fig, axs = plt.subplots(3, 1, sharex=True)
    axs[0].plot(x, loss(eps), label='Loss function')
    axs[0].plot(x, np.real(eps), label='Re( $\epsilon (\omega )$ )')
    axs[0].legend()
    axs[0].set_ylabel('Dielectric Function')

    axs[1].plot(x, surfaceLoss(eps), lineStyle='-')    
    axs[1].set_ylabel('Surface Loss Function')
    axs[1].set_ylim([0,None])

    axs[2].plot(x, reflectivity(eps))
    axs[2].set_ylabel('Reflectivity')
    axs[2].set_xlabel('Frequency')
    axs[2].set_ylim([0,1])
    axs[0].set_xlim(left=0, right=max(x))

    plt.show()

class oscillator:
    def __init__(self, wTO, wLO, gammaTO=20, gammaLO=20):
        self.wTO = wTO
        self.wLO = wLO
        self.gammaTO = gammaTO
        self.gammaLO = gammaLO

    def eps(self,w):
        nom = self.wLO*self.wLO - w*w + 1j*self.gammaLO*w
        denom = self.wTO*self.wTO - w*w + 1j*self.gammaTO*w
        return nom/denom

    def __call__(self, w):
        return self.eps(w)

class simpleOsci(oscillator):
    def __init__(self, wTO, Q, gammaTO=20):
        self.wTO = wTO
        self.Q = Q
        self.gammaTO = gammaTO

    def eps(self,w):
        nom = self.wTO*self.wTO*self.Q
        denom = self.wTO*self.wTO - w*w + 1j*self.gammaTO*w
        return nom/denom

class drude:
    def __init__(self, wPL, gamma):
        self.wPL = wPL
        self.gamma = gamma

    def eps(self,w):
        return 1. - (self.wPL*self.wPL)/(w*w - 1j*self.gamma*w)

    def __call__(self, w):
        return self.eps(w)

class drude2:
    '''This defines an additive Drude component to a dielectric function: 
    eps_drude = drude2(omega_p, gamma_p, gamma_0)
    ... eps(omega) + eps_drude(omega)
    '''
    def __init__(self, omega_p, gamma_p, gamma_0):
        self.omega_p = omega_p
        self.gamma_p = gamma_p
        self.gamma_0 = gamma_0

    def eps(self,w):
        newW = np.where(w==0, 0.000567894, w)   # Avoid zeros to avoid devision by zero 
        w = newW
        return -((self.omega_p**2 + 1j*(self.gamma_p-self.gamma_0)*w)/(w*(w - 1j *self.gamma_0)))

    def __call__(self, w):
        return self.eps(w)

def myMain():
    # BaTiO3
    data = [
        [178.6, 8.02, 3.09800E-02],
        [270.6, 24.00, 10.0800E-02],
        [522.9, 1.20, 6.97200E-02]]
    # Width (3rd parameter) is given in fraction of TO frequency

    oscis = [simpleOsci(TO, Q, f*TO) for (TO, Q, f) in data]
        
    x = np.linspace(0,1000,num=1200)
    epsInfinity = 5.25
    eps = epsInfinity
    for each in oscis:
        eps += each(x)

    plotDielectrics(x, eps)

if __name__ == '__main__':
	myMain()



#!/usr/bin/env python3
import os
import re
import sys

import matplotlib.pyplot as plt
import numpy as np
from matplotlib.backends.backend_qt5agg import \
    FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import \
    NavigationToolbar2QT as NavigationToolbar
from PyQt5 import QtCore, QtGui, QtWidgets, uic
import libhreels as hh
hhPath = os.path.dirname(__file__)
from libhreels.HREELS import HREELS, myPath
from datetime import datetime  
import argparse
# import libhreels.expLogbook as lgb
import libhreels.NoExpLogbook as lgb

# fix HighRes Displays
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)

class HREELS_Window(QtWidgets.QMainWindow):
    """ fill in some initial data """
    def __init__(self,datapath=None, remoteDir=None, startWithFile=None, comment=None):
        super(HREELS_Window, self).__init__()
        f = os.path.join(hhPath,"viewhreels.ui")  # path to the package directory. Needed to find graphical user interface files (*.ui)
        self.ui = self.ui = uic.loadUi(f, self)
        self.xmin = 50.
        self.factor = 10.
        self.offset = 0.1
        self.useOffset = False
        self.normalized = False
        self.comment = comment
        self.showComment = lgb.available
        if datapath and os.path.exists(datapath):
            self.datapath = datapath
        else:
            self.datapath = "./"
        self.directory = self.datapath
        if remoteDir and os.path.exists(remoteDir):
            self.remoteDir = remoteDir
        else:
            self.remoteDir = '//141.48.167.189/BackUp02/0_experiments/EELS-PC4/HREELS/gph_dat/'
        self.wideScan = True
        self.marker = []
        # self.markerSet = False

        # initialize the widget "drawWidget" and attach it into the ui
        self.drawing_figure = plt.figure(frameon=False,figsize=(5, 4), dpi=150, tight_layout = True)
        self.drawing_pane = FigureCanvas(self.drawing_figure)
        self.drawing_figure.canvas.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.drawing_figure.canvas.setFocus()        
        # self.drawing_figure.canvas.mpl_connect('button_press_event', self.onMouse)
        self.drawing_pane.axes = self.drawing_pane.figure.add_subplot(111)
        self.ui.drawWidget.layout().addWidget(self.drawing_pane)

        #Add a cursor:
        self.cursor = Cursor(self.drawing_pane, self.drawing_pane.axes)
        # self.cursorId = self.drawing_figure.canvas.mpl_connect('motion_notify_event', self.cursor.mouse_move)

        # Add toolbar
        self.xToolbar = NavigationToolbar(self.drawing_pane, self)
        self.ui.drawWidget.layout().addWidget(self.xToolbar)
        self.xToolbar.setSizePolicy(QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed))
        
        self.ui.lineEdit_directory.setText(self.datapath)
        self.ui.lineEdit_xmin.setText('%4.1f'%self.xmin)
        self.ui.lineEdit_factor.setText('%4.1f'%(self.factor))
        self.ui.lineEdit_offset.setText('%4.1f'%(self.offset))
        self.ui.checkBoxWideScans.setChecked(True)
        self.ui.checkBoxComment.setChecked(lgb.available)
        # self.ui.listWidget.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)


        # Action on user events
        self.ui.checkBoxWideScans.toggled.connect(self.onWideScans)
        self.ui.checkBoxNormalized.toggled.connect(self.onNormalized)
        self.ui.checkBoxOffset.toggled.connect(self.onUseOffset)
        self.ui.checkBoxMarker.toggled.connect(self.onMarkerSet)
        self.ui.checkBoxComment.toggled.connect(self.onCommentSet)
        self.ui.pushButton_details.clicked.connect(self.onPushButton_details)
        self.ui.lineEdit_xmin.editingFinished.connect(self.onXmin)
        self.ui.lineEdit_factor.editingFinished.connect(self.onFactor)
        self.ui.lineEdit_offset.editingFinished.connect(self.onOffset)
        self.ui.lineEdit_directory.editingFinished.connect(self.onNewDirectory)
        self.ui.listWidget.itemSelectionChanged.connect(self.onFileSelection)

        # Action on user menu events: 
        self.action_directory.triggered.connect(self.onActionDir)
        self.action_test.triggered.connect(self.onActionTest)
        self.action_help.triggered.connect(self.onActionHelp)
        self.actionFile_info.triggered.connect(self.onPushButton_details)

        # Create list of all datafiles in the directory
        self.createFileList()
        self.dataDir = QtWidgets.QFileDialog(self)
        if os.name == 'nt':
            self.dataDir.setDirectory("D:\\Data\\Python\\HREELS\\expHREELS\\data")
        else:
            self.dataDir.setDirectory("/mnt/d/Data/Python/HREELS/expHREELS/data")

        if startWithFile:
            self.selectFile(startWithFile)


    def onXmin(self):
        '''Routine called by a new value of "Xmin". It will trigger a data window update.'''
        try:
            val = float(self.ui.lineEdit_xmin.text())
            self.xmin = val
        except ValueError:
            self.ui.lineEdit_xmin.setText('%4.1f'%self.xmin)
        self.onFileSelection()

    def onFactor(self):
        '''Routine called by a new value of "Factor", which defines the amplification factor for the energy loss
        region (larger than Xmin). It will trigger a data window update.'''
        try:
            val = float(self.ui.lineEdit_factor.text())
            self.factor = val
        except ValueError:
            self.ui.lineEdit_factor.setText('%4.1f'%(self.factor))
        self.onFileSelection()

    def onUseOffset(self):
        '''Routine called by toggling "Use Offset": If set, the  energy loss
        region (larger than Xmin) will be vertically offseted if multiple spectra are selected. 
        It will trigger a data window update.'''
        self.useOffset = self.ui.checkBoxOffset.isChecked()
        self.onOffset()

    def onOffset(self):
        try:
            val = float(self.ui.lineEdit_offset.text())
            self.offset = val
        except ValueError:
            self.ui.lineEdit_offset.setText('%4.1f'%(self.offset))
        self.onFileSelection()

    def onNormalized(self):
        '''Routine called by toggling "Normalized": If set, all spectra will be normalized to a maximum
        count rate of unity. It will trigger a data window update.'''
        self.normalized = self.ui.checkBoxNormalized.isChecked()
        self.onFileSelection()

    def onActionDir(self):
        '''Routine calls Directory Selection Interface upon Menu selection. It will 
        create a data list from the chosen directory.'''
        # if os.name == 'nt':
        #     self.dataDir.setDirectory("D:\\Data\\Python\\HREELS\\expHREELS\\data")
        # else:
        #     self.dataDir.setDirectory("/mnt/d/Data/Python/HREELS/expHREELS/data")
        directory = self.openDirectoryDialog()
        if directory:
            self.ui.lineEdit_directory.setText(directory)
            self.directory = directory
            self.ui.listWidget.clear()
            self.datapath = directory
            self.createFileList()

    def onNewDirectory(self):
        '''Routine called if Directory field is modified. It will 
        create a data list from the chosen new directory (but only if it exists).'''
        dirText = self.ui.lineEdit_directory.text()
        tt = myPath(dirText)
        print('New directory: ',tt)
        self.directoryOld = self.directory
        if os.path.exists(tt):
            self.directory = tt
            self.ui.listWidget.clear()
            self.datapath = self.directory
            self.createFileList()
        else:
            self.ui.lineEdit_directory.setText(self.directoryOld)


    def onActionTest(self):
        '''Routine called by Menu item 'Test'. It is for playing 
        with future options.'''
        self.dataDir.setDirectory(self.remoteDir)
        directory = self.openDirectoryDialog()
        if directory:
            self.ui.lineEdit_directory.setText(directory)
            self.directory = directory
            self.ui.listWidget.clear()
            self.datapath = directory
            self.createFileList()

    def onActionHelp(self):
        pass
        msg = QtWidgets.QMessageBox()
        msg.setText('''
        This is the HREELS data browser of the Martin-Luther University Halle-Wittenberg
        (Version 0.9) designed for fast data screening and plotting.
        Copyright @ wolf.widdra@physik.uni-halle.de.
        ''')
        msg.setWindowTitle("HREELS data browser help")
        # msg.setStandardButtons(QtWidgets.QMessageBox.Ok)
        layout = msg.layout()
        widget = QtWidgets.QWidget()
        widget.setFixedSize(650, 1)
        layout.addWidget(widget, 3, 0, 1, 3)
        msg.buttonClicked.connect(self.onOK)            
        msg.exec_()

    def onPushButton_details(self):
        """Routine for showing a dialog window containing HREELS data details"""
        try:
            self.d.infoText()
        except:
            return
        self.dialog = QtWidgets.QDialog(self)
        f = os.path.join(hhPath,"viewhreels_dialog.ui")
        self.dialog.ui = uic.loadUi(f, self.dialog)
        self.dialog.ui.setWindowTitle('Dataset details')
        self.dialog.ui.textArea.setPlainText(self.d.infoText())
        self.dialog.show()

    def onMarkerSet(self):
        '''Routine called if selection "Set Marker" is toggled. It allows to draw dashed vertical lines
        with the energy loss values. The value 'self.markerSet' is used to decide later if lines have to be drawn.'''
        b = self.ui.checkBoxMarker.isChecked()
        self.markerSet = b
        if not b:
            # Disconnect all mouse events from canvas:
            self.drawing_figure.canvas.mpl_disconnect(self.cursorId)
            self.drawing_figure.canvas.mpl_disconnect(self.canvasId)
        else:
            # Connect all mouse events from canvas to routines:
            self.cursorId = self.drawing_figure.canvas.mpl_connect('motion_notify_event', self.cursor.mouse_move)
            self.canvasId = self.drawing_figure.canvas.mpl_connect('button_press_event', self.onMouse)
            print('mouse enabled')

    def onCommentSet(self):
        '''Routine called if selection "Show Comment" is toggled. It allows to draw dashed vertical lines
        with the energy loss values. The value 'self.markerSet' is used to decide later if lines have to be drawn.'''
        self.showComment =  self.ui.checkBoxComment.isChecked()
        # print('onCommentSet:', lgb.available, self.showComment, self.ui.listWidget.selectedItems()[0].text())
        if self.showComment:
            if lgb.available:
                text = lgb.logbook.getShortMessage4File(self.d.fname)
                ids = lgb.logbook.search({'file':self.d.fname})
                _, attributes, _ = lgb.logbook.read(ids[0])
                print(self.d.fname, ids[0],':\t\t', attributes['Date'],'\t',text)
            else:
                text = self.comment if self.comment else ''
                self.comment = None
            self.pCom = self.drawing_pane.axes.text(0.12, 0.98,text, ha='left', va='top', transform=self.drawing_pane.axes.transAxes)
        else:
            try:
                self.pCom.remove()
            except:
                pass

    def onOK(self, button):
        print("Button pressed is:",button.text())

    def onWideScans(self):
        '''Routine called if selection "Only wide scans" is toggled. The switch 'self.wideScan' decides 
        if all spectra or only spectra with a wider energy range (data files larger than 21 kB) 
        are listed on the left side.'''
        b = self.ui.checkBoxWideScans.isChecked()
        self.wideScan = b
        self.ui.listWidget.clear()
        self.createFileList()

    def selectFile(self, file):
        '''Select the file within the QListWidget by the program, e.g. as start parameter. 
        Make sure before that it exists. The file extension is dropped by re.split. '''
        matchingItems = self.ui.listWidget.findItems(re.split('\.',file)[0], QtCore.Qt.MatchExactly)
        self.ui.listWidget.setCurrentItem(matchingItems[0])
        for it in matchingItems:
            it.setSelected(True)
        self.ui.listWidget.setFocus()
        return

    def onFileSelection(self):
        '''Routine called if user makes a file selection or changes the file selection within the left list.
        The selected spectra are plotted according to the given flags (Xmin, Factor, Normalized, Offset). '''
        iMax = len(self.ui.listWidget.selectedItems())
        if iMax == 0:
            return
        # print([it.text() for it in self.ui.listWidget.selectedItems()])
        firstItem = self.ui.listWidget.selectedItems()[0]

        self.d = HREELS(firstItem.text(),self.datapath)
        if self.d.valid:
            plotColor = plt.cm.gnuplot(0)     # Define the color map (coolwarm; plasma; inferno; gnuplot)         
            self.updatePlot(xmin=self.xmin, factor=self.factor, normalized=self.normalized, color=plotColor)
            try:
                self.dialog.ui.textArea.setPlainText(self.d.infoText())
            except:
                pass
        else:
            self.drawing_pane.axes.cla()
            self.drawing_pane.draw()
            return
        if iMax > 1:
            i = 1.
            myList = self.ui.listWidget.selectedItems()[1:]
            for item in myList:     # Add here sorting according to ...
                plotColor = plt.cm.gnuplot(i/iMax)     # Define the color map (coolwarm; plasma; inferno; gnuplot)
                self.d = HREELS(item.text(),self.datapath)
                self.secondPlot(xmin=self.xmin, factor=self.factor, normalized=self.normalized, offset=i*self.offset*self.useOffset, color=plotColor)
                # print(i, item.text())
                i += 1
        # Reversing the sequence of labels in the legend:
        handles, labels = self.drawing_pane.axes.get_legend_handles_labels()
        self.drawing_pane.axes.legend(handles[::-1], labels[::-1])
        self.drawing_pane.axes.set_ylim(bottom=0.)
        self.drawing_figure.tight_layout()
        self.cursor = Cursor(self.drawing_pane, self.drawing_pane.axes)
        self.drawing_figure.canvas.mpl_connect('motion_notify_event', self.cursor.mouse_move)
        self.drawing_pane.draw()

    def onMouse(self, event):
        if not self.markerSet:
            return
        if (event.xdata):
            if event.button==1:
                x,y = event.xdata, event.ydata
                print(x,y)
                self.marker.append((x,y))
                self.setMarker(x,y)
            elif event.button==3:
                pass
            self.drawing_pane.draw()

    def setMarker(self, x, y, ymin=0, size=None):
        ''' Set vertical marker with text label. Note that self.figure() needs to be 
        called before use.'''
        # trans = offset(self.ax, 0, 30)
        self.drawing_pane.axes.plot([x,x],[ymin,y], lw=1, c='black', ls='dashed')
        x = round(x)
        # plt.text(x,y,'%4i' % x, rotation=90, verticalalignment='bottom', horizontalalignment='center', transform=trans)
        self.drawing_pane.axes.text(x,y,'%4i' % x, rotation=90, verticalalignment='bottom', horizontalalignment='center')
        return
    

    def updatePlot(self,xmin=70, factor=10, normalized=False, color="red"):
        '''Plotting the first of the selected spectra. '''
        self.drawing_pane.axes.cla()
        self.plotWidget(normalized=normalized,showComment=False)
        self.plotWidget(xmin=xmin, factor=factor, color=color, label='x %3i'%factor, normalized=normalized, showComment=self.showComment)

    def secondPlot(self,xmin=70, factor=10, normalized=False, offset = 0., color="b-"):
        '''Plotting the second and all further selected spectra. '''
        self.plotWidget(xmin=xmin, factor=factor, color=color, label=self.d.fname, normalized=normalized, offset = offset)

    def plotWidget(self, xmin=None, xmax=None, factor=1, label='x', normalized=False, color="black",
                        marker=True, offset = 0., showComment=False):
        ''' plot(self, xmin=None, xmax=None, factor=1, label='x', normalized=False, color="b-",marker=True)'''
        self.d.pFactor = factor
        if xmin:
            nstart = self.d.findIndex(xmin)
        else:
            nstart = 0
        if xmax:
            nend = self.d.findIndex(xmax)
        else:
            nend = len(self.d.xdata)
        if normalized:
            factor /= self.d.maxIntensity
        if label == "x":
            label = self.d.fname        
        self.drawing_pane.axes.plot(self.d.xdata[nstart:nend], factor*self.d.ydata[nstart:nend] + offset, color=color, label=label)
        self.d.factor = factor
        if showComment: self.onCommentSet()
        self.drawing_pane.axes.set_xlabel('Energy Loss (cm$^{-1}$)')
        if normalized:
            self.drawing_pane.axes.set_ylabel('Normalized Intensity')
        else:
            self.drawing_pane.axes.set_ylabel('Intensity (s$^{-1}$)')
        if marker:
            for (x,y) in self.d.marker:
                self.d.setMarker(x,y*factor)
        return

    def getFileList(self, directory, sizeMin=0.):
        '''Get the list of all *.gph or *.GPH data files with a file size larger 
        than sizeMin. A good value to remove the datfile with just the elastic peak is 
        sizeMin=21000. The list is sorted by the file creation date.'''
        list = os.listdir(directory)
        files = []
        for file in list:
            location = os.path.join(directory, file)
            size = os.path.getsize(location)
            time = os.path.getmtime(location)
            if size>sizeMin:
                files.append((file, time, size))
        # files.sort(key=lambda s: -s[1])  # sort by time
        files.sort(key=lambda s: s[1])  # sort by reversed time
        fileNames = [item[0] for item in files]
        data_ext = ['gph', 'GPH']
        gphFiles = [fn for fn in fileNames
                    if any(fn.endswith(ext) for ext in data_ext)]
        return gphFiles, len(gphFiles)

    def createFileList(self):
        if self.wideScan:
            sizeMin = 21000.
        else:
            sizeMin = 5.
        for item in self.getFileList(self.datapath,sizeMin=sizeMin)[0]:
            fname = re.split('\.',item)[0]
            self.ui.listWidget.addItem(fname)
          
    def openDirectoryDialog(self):
        self.dataDir = QtWidgets.QFileDialog(self)
        options = self.dataDir.Options()
        # options = self.dataDir.ShowDirsOnly
        options |= self.dataDir.DontUseNativeDialog
        dir =self.dataDir.getExistingDirectory(self, 'Select data directory', options=options)
        if dir:
            return dir

class Cursor(object):
    def __init__(self, fig, ax):
        self.figure = fig
        self.ax = ax
        self.ly = ax.axvline(color='r')  # the vert line

    def mouse_move(self, event):
        if not event.inaxes:
            return
        self.ly.set_data([event.xdata, event.xdata], [0, 1])
        self.figure.draw()

def runViewer(datapath= None, remoteDir=None, startWithFile=None, comment=None):
    """ Main function for graphical application start """
    app = QtWidgets.QApplication(sys.argv)
    form = HREELS_Window(datapath=datapath, remoteDir=remoteDir, startWithFile=startWithFile, comment=comment)
    form.show()
    sys.exit(app.exec_())


def myMain():
    parser = argparse.ArgumentParser()
    #################  Command line handling  ######################
    def is_valid_file(string):
        if not os.path.exists(string):
            parser.error("The file %s does not exist!" % string)
            return False
        else:
            return string
    # parser.add_argument("-s", "--start", help="Start date (2018-01-01)", default='2019-01-01', type=validDate)
    parser.add_argument("-f", "--file", help="Start viewer with this datafile", type=is_valid_file)
    # parser.add_argument("-f", "--file", help="Start viewer with this datafile")
    parser.add_argument("-c", "--comment", default=None, help="Text comment")
    args = parser.parse_args()

    if args.file:
        fPath,fName = os.path.split(args.file)
        # print(">>> ", fPath, " <<< ", fName)
    else:
        if os.name == 'nt':
            fPath = "D:\\Data\\Python\\HREELS\\expHREELS\\data"
        else:
            fPath = "/mnt/d/Data/Python/HREELS/expHREELS/data"
        fName = None

    runViewer(datapath=fPath, startWithFile=fName, comment=args.comment)




if __name__ == '__main__':
	myMain()


#!/usr/bin/python
# Auger library Version 0.1    (December 2018 @ WFW)
#
import numpy as np
import re
import matplotlib.pyplot as plt
from scipy import optimize, interpolate
from datetime import datetime
from datetime import timedelta
from matplotlib.transforms import offset_copy
# from pylab import figure
from matplotlib.pyplot import figure
import os

def offset(ax, x, y):
    return offset_copy(ax.transData, x=x, y=y, units='dots')

def Auger_elog_Dictionary(file, path):
    spec = Auger(file,datapath=path)
    dic = {'Category':'Auger', 'Type':'Auger', 'Substrate':'as before', 
        'Summary':'%5.0f'% spec.xmin +'-'+'%-5.0f'% spec.xmax, 
        'Temperature':'', 'File':str(file), 
        'Scan_Range':'%5.0f'% spec.xmin +','+'%5.0f'% spec.xmax, 
        'N_channels':'%4d'%spec.totalTime, 'MDate':spec.startTimeEpoch, 
        'Author':'Python Script'}
    return dic

class Auger:
    """This class handles Auger data and their plotting."""
    path = "./"
    def __init__(self, _filename, datapath=path):
        self.datapath = datapath
        self.readDataFile(_filename)
        self.Auger = interpolate.interp1d(self.xdata, self.ydata, kind='linear')
        # self.Auger = interp1d(self.xdata, self.ydata, kind='nearest')

    def readDataFile(self, _filename):
        # if _filename.find(".") == -1:
        #     _filename +=".aes"
        self.filename = _filename
        # self.fname = self.filename[0:self.filename.find(".")]
        self.fname = _filename
        xdata = []
        ydata = []
        import glob
        f = glob.glob(self.datapath+"**/"+self.filename, recursive=True)
        if len(f) == 0:
            print('Error: No files found!')
            print('Not found:',self.filename)
            return
        f = glob.glob(self.datapath+"**/"+self.filename, recursive=True)[0]
        self.datapath = os.path.dirname(f)
        self.startTime = datetime.fromtimestamp(os.path.getmtime(f))        # Only the file saving time
        self.startTimeEpoch = os.path.getmtime(f)
        with open(f, 'r') as f:
            for line in f:
                str_x, str_d, str_y = line.split()
                # print(str_x, str_y)
                try:
                    xdata.append(float(str_x))
                    ydata.append(float(str_y))
                except:
                    xdata.append(float(str_x.replace(",",".")))
                    ydata.append(float(str_y.replace(",",".")))
        self.xdata = np.array(xdata)
        self.ydata = np.array(ydata)
        self.xmin = self.xdata[0]
        self.xmax = self.xdata[-1]
        self.deltay = max(ydata)-min(ydata)
        self.ymax = max(ydata)
        self.totalTime = 1.0 * len(self.xdata)
        self.marker = []
        self.valid = True


    def __str__(self):
        return self.datapath+self.filename

    def info(self):
        print("filename: \t\t",self.filename)
        print("datapath: \t\t",self.datapath)
        print("startTime: \t\t",self.startTime)
        print("xmin: \t\t\t","%6.1f"%self.xmin,"eV")
        print("xmax: \t\t\t","%6.1f"%self.xmax,"eV")
        if len(self.marker)>0:
            print("marker: \t\t")
            for each in self.marker:
                print("\t\t%6.1f\t %8.1f"%each)
        print('--------')

    def infoText(self):
        langtext = "filename: \t\t"+self.filename+'\n'
        langtext +="datapath: \t\t"+self.datapath+'\n'
        langtext +="startTime: \t\t"+str(self.startTime)+'\n'
        langtext +="xmin: \t\t\t"+"%6.1f"%self.xmin+" eV"+'\n'
        langtext +="xmax: \t\t\t"+"%6.1f"%self.xmax+" eV"+'\n'
        if len(self.marker)>0:
            langtext +="marker: \t\t"+'\n'
            for each in self.marker:
                langtext +="\t\t%6.1f\t %8.1f"%each+'\n'
        langtext +='--------'
        return langtext


    def findIndex(self, kinEnergy):
        for i in range(len(self.xdata)):
            if self.xdata[i] > kinEnergy: 
                return i
        print('Auger.findIndex: kinEnergy', kinEnergy, 'not found in xdata')
        return len(self.xdata)-1

    def plot(self, xmin=None, xmax=None, label='x', color="b-",marker=True, offset=0.):
        ''' plot(self, xmin=None, xmax=None,  label='x', normalized=False, color="b-",marker=True,offset=0.)'''
        if xmin:
            nstart = self.findIndex(xmin)
        else:
            nstart = 0
        if xmax:
            nend = self.findIndex(xmax)
        else:
            nend = len(self.xdata)
        if label == "x":
            label = self.filename
        plt.plot(self.xdata[nstart:nend], self.ydata[nstart:nend]+offset, color, label=label)
        plt.xlabel('Kinetic Energy (eV)')
        plt.ylabel('dN/dE (arb. u.)')
        plt.legend()
        if marker:
            for (x,y) in self.marker:
                self.setMarker(x,y)
        return

    def pickMarker(self):
        def onclick(event):
            # print('%s click: button=%d, x=%d, y=%d, xdata=%f, ydata=%f' %
            #     ('double' if event.dblclick else 'single', event.button,
            #     event.x, event.y, event.xdata, event.ydata))
            if event.button ==1:
                self.marker.append((event.xdata, event.ydata))
                self.setMarker(event.xdata, event.ydata)
                self.fig.canvas.draw()
            else:
                self.marker = []
                print("All marker deleted!")
                self.fig.canvas.draw()
        self.fig.canvas.mpl_connect('button_press_event', onclick)
        self.ax.set_title('Pick marker with left mouse button ... (right:erase all)')

    def figure(self):
        self.fig=figure()
        self.ax=self.fig.add_subplot(111)

    def show(self):
        plt.show()

    def selectData(self,x1, x2=None):
        """Returns the data as (x,y) list between 
         x1 and x2. If x2 is omitted the full range is used."""
        if not x2:
            x2 = self.xmax
        a = self.findIndex(x1)
        b = self.findIndex(x2)+1
        return self.data[a:b]

    def setMarker(self, x, y, ymin=0, size=None):
        trans = offset(self.ax, 0, 10)
        plt.plot([x,x],[y,self.ymax-0.2*self.deltay], lw=1, c='b', ls='dashed')
        x = round(x*10)/10.
        plt.text(x,self.ymax-0.2*self.deltay,'%5.1f' % x, rotation=90, 
        verticalalignment='bottom', horizontalalignment='center', transform=trans)
        return


if __name__ == '__main__':
    path = 'D:\Data\Python\Auger\data'
    d1 = Auger("181207_A_01",datapath=path)
    d2 = Auger("190207_A_02",datapath=path)
    dy=0.
    d2.info();d2.plot(color='m-');dy-=0.5
    d1.info();d1.plot(offset=dy);dy-=0.5
    d2.show()

    import code
    code.interact(local=locals())



#!/usr/bin/env python3
import os
import re
import sys

import matplotlib.pyplot as plt
import numpy as np
from datetime import datetime  
import argparse
from matplotlib.backends.backend_qt5agg import \
    FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import \
    NavigationToolbar2QT as NavigationToolbar
from PyQt5 import QtCore, QtGui, QtWidgets, uic
import libhreels as hh
hhPath = hh.__path__[0]
from libhreels.Auger import Auger

# fix HighRes Displays
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class Auger_Window(QtWidgets.QMainWindow):
    """ fill in some initial data """

    def __init__(self,datapath= None, remoteDir=None, startWithFile=None):
        super(Auger_Window, self).__init__()
        f = os.path.join(hhPath,"viewauger.ui")
        self.ui = self.ui = uic.loadUi(f, self)
        self.xmin = 10.
        self.factor = 1.
        self.offset = 0.5
        self.useOffset = False
        self.normalized = False
        if datapath and os.path.exists(datapath):
            self.datapath = datapath
        else:
            self.datapath = "\\\\141.48.167.189\\BackUp02\\0_experiments\\EELS-PC4\\AES"
        if remoteDir and os.path.exists(remoteDir):
            self.remoteDir = remoteDir
        else:
            self.remoteDir = '\\\\141.48.167.189\\BackUp02\\0_experiments\\EELS-PC4\\Auger'
        self.wideScan = True
        self.marker = []
        # self.markerSet = False

        # initialize the widget "drawWidget" and attach it into the ui
        self.drawing_figure = plt.figure(frameon=False,figsize=(5, 4), dpi=150, tight_layout = True)
        self.drawing_pane = FigureCanvas(self.drawing_figure)
        self.drawing_figure.canvas.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.drawing_figure.canvas.setFocus()        
        # self.drawing_figure.canvas.mpl_connect('button_press_event', self.onMouse)
        self.drawing_pane.axes = self.drawing_pane.figure.add_subplot(111)
        self.ui.drawWidget.layout().addWidget(self.drawing_pane)
        # Add toolbar
        self.xToolbar = NavigationToolbar(self.drawing_pane, self)
        self.ui.drawWidget.layout().addWidget(self.xToolbar)
        self.xToolbar.setSizePolicy(QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed))
        
        self.ui.lineEdit_directory.setText(self.datapath)
        self.ui.lineEdit_factor.setText('%4.1f'%(self.factor))
        self.ui.lineEdit_offset.setText('%4.1f'%(self.offset))

        # Action on user events
        self.ui.checkBoxOffset.toggled.connect(self.onUseOffset)
        self.ui.checkBoxMarker.toggled.connect(self.onMarkerSet)
        self.ui.pushButton_details.pressed.connect(self.onPushButton_details3)
        self.ui.lineEdit_factor.editingFinished.connect(self.onFactor)
        self.ui.lineEdit_offset.editingFinished.connect(self.onOffset)
        self.ui.lineEdit_directory.editingFinished.connect(self.onNewDirectory)

        # Create list of all datafiles in the directory
        self.createFileList()
        self.ui.listWidget.itemSelectionChanged.connect(self.onFileSelection)
        self.dataDir = QtWidgets.QFileDialog(self)
        if os.name == 'nt':
            self.dataDir.setDirectory("D:\\Data\\Python\\HREELS\\expHREELS\\data")
        else:
            self.dataDir.setDirectory("/mnt/d/Data/Python/HREELS/expHREELS/data")

        # Action on user menu events: 
        self.action_directory.triggered.connect(self.onActionDir)
        self.action_test.triggered.connect(self.onActionTest)
        self.action_help.triggered.connect(self.onActionHelp)

        #Add a cursor:
        self.cursor = Cursor(self.drawing_pane, self.drawing_pane.axes)
        # self.cursorId = self.drawing_figure.canvas.mpl_connect('motion_notify_event', self.cursor.mouse_move)
        if startWithFile:
            self.selectFile(startWithFile)



    def onFactor(self):
        try:
            val = float(self.ui.lineEdit_factor.text())
            self.factor = val
        except ValueError:
            self.ui.lineEdit_factor.setText('%4.1f'%(self.factor))
        self.onFileSelection()

    def onUseOffset(self):
        self.useOffset = self.ui.checkBoxOffset.isChecked()
        self.onOffset()

    def onOffset(self):
        try:
            val = float(self.ui.lineEdit_offset.text())
            self.offset = val
        except ValueError:
            self.ui.lineEdit_offset.setText('%4.1f'%(self.offset))
        self.onFileSelection()

    def onNormalized(self):
        self.normalized = self.ui.checkBoxNormalized.isChecked()
        self.onFileSelection()

    def onActionDir(self):
        # self.dataDir.setDirectory("D:\\Data\\Python\\expHREELS\\data")
        directory = self.openDirectoryDialog()
        if directory:
            self.ui.lineEdit_directory.setText(directory)
            self.directory = directory
            self.ui.listWidget.clear()
            self.datapath = directory
            self.createFileList()

    def onNewDirectory(self):
            dirText = self.ui.lineEdit_directory.text()
            if os.path.exists(dirText):
                self.directory = self.ui.lineEdit_directory.text()
                self.ui.listWidget.clear()
                self.datapath = self.directory
                self.createFileList()
            else:
                self.ui.lineEdit_directory.setText('')


    def onActionTest(self):
        self.dataDir.setDirectory(self.remoteDir)
        directory = self.openDirectoryDialog()
        if directory:
            self.ui.lineEdit_directory.setText(directory)
            self.directory = directory
            self.ui.listWidget.clear()
            self.datapath = directory
            self.createFileList()

    def onActionHelp(self):
        pass
        msg = QtWidgets.QMessageBox()
        msg.setText('''
        This is the Auger data browser of the Martin-Luther University Halle-Wittenberg
        (Version 0.5) designed for fast data screening and plotting.
        Copyright @ wolf.widdra@physik.uni-halle.de.
        ''')
        msg.setWindowTitle("Auger data browser help")
        # msg.setStandardButtons(QtWidgets.QMessageBox.Ok)
        layout = msg.layout()
        widget = QtWidgets.QWidget()
        widget.setFixedSize(650, 1)
        layout.addWidget(widget, 3, 0, 1, 3)
        msg.buttonClicked.connect(self.onOK)            
        msg.exec_()

    def onPushButton_details(self):
        try:
            details = self.d.infoText()
        except:
            return
        msg = QtWidgets.QMessageBox()
        msg.setText(details)
        msg.setWindowTitle("Dataset details")
        msg.setStandardButtons(QtWidgets.QMessageBox.Ok)
        layout = msg.layout()
        widget = QtWidgets.QWidget()
        widget.setFixedSize(650, 1)
        layout.addWidget(widget, 3, 0, 1, 3)
        msg.buttonClicked.connect(self.onOK)            
        msg.exec_()

    def onPushButton_details3(self):
        """Shows a dialog containing data details"""
        self.dialog = QtWidgets.QDialog(self)
        f = os.path.join(hhPath, "viewhreels_dialog.ui")
        self.dialog.ui = uic.loadUi(f, self.dialog)
        self.dialog.ui.setWindowTitle('Dataset details')
        self.dialog.ui.textArea.setPlainText(self.d.infoText())
        self.dialog.show()

    def onMarkerSet(self):
        b = self.ui.checkBoxMarker.isChecked()
        self.markerSet = b
        if not b:
            # Disconnect all mouse events from canvas:
            self.drawing_figure.canvas.mpl_disconnect(self.cursorId)
            self.drawing_figure.canvas.mpl_disconnect(self.canvasId)
        else:
            # Connect all mouse events from canvas to routines:
            self.cursorId = self.drawing_figure.canvas.mpl_connect('motion_notify_event', self.cursor.mouse_move)
            self.canvasId = self.drawing_figure.canvas.mpl_connect('button_press_event', self.onMouse)
            print('mouse enabled')

    def onOK(self, button):
        print("Button pressed is:",button.text())

        
    def selectFile(self, file):
        '''Select the file within the QListWidget by then program, e.g. as start parameter. 
        Make sure before that it exists. The file extension is dropped by re.split. '''
        itm = self.ui.listWidget.findItems(re.split('\.',file)[0], QtCore.Qt.MatchExactly)
        self.ui.listWidget.setCurrentItem(itm[0])
        return


    def onFileSelection(self):
        iMax = len(self.ui.listWidget.selectedItems())
        if iMax == 0:
            return
        firstItem = self.ui.listWidget.selectedItems()[0]
        self.d = Auger(firstItem.text(),self.datapath)
        if iMax > 1:        # Remember the last plot size if there are already plots
            # Read xmin, xmax for current window and set these values for added spectra:
            self.xdmin, self.xdmax = self.drawing_pane.axes.get_xlim() 
        plotColor = plt.cm.gnuplot(0)     # Define the color map (coolwarm; plasma; inferno; gnuplot)         
        self.updatePlot(xmin=self.xmin, factor=self.factor, normalized=self.normalized, color=plotColor)
        self.ydmin, _ = self.drawing_pane.axes.get_ylim() 

        try:
            self.dialog.ui.textArea.setPlainText(self.d.infoText())
        except:
            pass

        if iMax > 1:
            i = 1.
            for item in self.ui.listWidget.selectedItems()[1:]:
                plotColor = plt.cm.gnuplot(i/iMax)     # Define the color map (coolwarm; plasma; inferno; gnuplot)
                self.d = Auger(item.text(),self.datapath)
                self.secondPlot(xmin=self.xmin, factor=self.factor, normalized=self.normalized, offset=i*self.offset*self.useOffset, color=plotColor)
                # print(i, item.text())
                i += 1
        # Reversing the sequence of labels in the legend:
        handles, labels = self.drawing_pane.axes.get_legend_handles_labels()
        self.drawing_pane.axes.legend(handles[::-1], labels[::-1])
        # self.drawing_pane.axes.set_ylim(bottom=0.)
        self.drawing_figure.tight_layout()
        self.cursor = Cursor(self.drawing_pane, self.drawing_pane.axes)
        self.drawing_figure.canvas.mpl_connect('motion_notify_event', self.cursor.mouse_move)
        self.drawing_pane.draw()

    def onMouse(self, event):
        if not self.markerSet:
            return
        if (event.xdata):
            if event.button==1:
                x,y = event.xdata, event.ydata
                print(x,y)
                self.marker.append((x,y))
                self.setMarker(x,y,self.ydmin)
            elif event.button==3:
                pass
            self.drawing_pane.draw()

    def setMarker(self, x, y, ymin=0, size=None):
        ''' Set vertical marker with text label. Note that self.figure() needs to be 
        called before use.'''
        # trans = offset(self.ax, 0, 30)
        self.drawing_pane.axes.plot([x,x],[ymin,y], lw=1, c='black', ls='dashed')
        x = round(x)
        # plt.text(x,y,'%4i' % x, rotation=90, verticalalignment='bottom', horizontalalignment='center', transform=trans)
        self.drawing_pane.axes.text(x,y,'%4i' % x, rotation=90, verticalalignment='bottom', horizontalalignment='center')
        return
    

    def updatePlot(self,xmin=10, factor=1, normalized=False, color="red"):
        self.drawing_pane.axes.cla()
        # self.plotWidget(normalized=normalized)
        self.plotWidget(factor=factor, color=color, normalized=normalized)

    def secondPlot(self,xmin=70, factor=1, normalized=False, offset = 0., color="b-"):
        self.secondPlotWidget(xmin=xmin, xdmin=self.xdmin, xdmax=self.xdmax, factor=factor, color=color, label=self.d.fname, normalized=normalized, offset = offset)

    def plotWidget(self, xmin=None, xmax=None, factor=1, label='x', normalized=False, color="black",marker=True, offset = 0.):
        ''' plot(self, xmin=None, xmax=None, factor=1, label='x', normalized=False, color="b-",marker=True)'''
        self.d.pFactor = factor
        if xmin:
            nstart = self.d.findIndex(xmin)
        else:
            nstart = 0
        if xmax:
            nend = self.d.findIndex(xmax)
        else:
            nend = len(self.d.xdata)
        if label == "x":
            label = self.d.fname        
        self.drawing_pane.axes.plot(self.d.xdata[nstart:nend], factor*self.d.ydata[nstart:nend] + offset, color=color, label=label)
        self.d.factor = factor
        self.drawing_pane.axes.set_xlabel('Kinetic Energy (eV)')
        self.drawing_pane.axes.set_ylabel('Intensity (arb. units)')
        if marker:
            for (x,y) in self.d.marker:
                self.d.setMarker(x,y*factor)
        return

    def secondPlotWidget(self, xmin=None, xmax=None, xdmin=None, xdmax=None, factor=1, label='x', normalized=False, color="black",marker=True, offset = 0.):
        ''' plot(self, xmin=None, xmax=None, factor=1, label='x', normalized=False, color="b-",marker=True)'''
        self.d.pFactor = factor
        if xmin:
            nstart = self.d.findIndex(xmin)
        else:
            nstart = 0
        if xmax:
            nend = self.d.findIndex(xmax)
        else:
            nend = len(self.d.xdata)
        if label == "x":
            label = self.d.fname        
        self.drawing_pane.axes.plot(self.d.xdata[nstart:nend], factor*self.d.ydata[nstart:nend] + offset, color=color, label=label)
        self.drawing_pane.axes.set_xlim(xdmin, xdmax) 
        self.d.factor = factor
        return

    def getFileList(self, directory):
        '''Get the list of all Auger data files. The list is 
        sorted by the file creation date.'''
        list = os.listdir(directory)
        files = []
        for file in list:
            location = os.path.join(directory, file)
            size = os.path.getsize(location)
            time = os.path.getmtime(location)
            files.append((file, time, size))
        files.sort(key=lambda s: -s[1])  # sort by time
        fileNames = [item[0] for item in files]
        # SElect the specific Auger data files: E.g. 181207_A_01
        # AESFiles = [fn for fn in fileNames if (fn[6:9]=='_A_')]
        AESFiles = []
        for fn in fileNames:
            if (fn[6:9]=='_A_'):
                AESFiles.append(fn)
            elif re.search('^[1-3][0-9][0-1][0-9][0-3][0-9]_', fn):
                AESFiles.append(fn)
        return AESFiles, len(AESFiles)

    def createFileList(self):
        for item in self.getFileList(self.datapath)[0]:
            fname = re.split('\.',item)[0]
            self.ui.listWidget.addItem(fname)
          
    def openDirectoryDialog(self):
        self.dataDir = QtWidgets.QFileDialog(self)
        options = self.dataDir.Options()
        # options = self.dataDir.ShowDirsOnly
        options |= self.dataDir.DontUseNativeDialog
        dir =self.dataDir.getExistingDirectory(self, 'Select data directory', options=options)
        if dir:
            return dir

class Cursor(object):
    def __init__(self, fig, ax):
        self.figure = fig
        self.ax = ax
        self.ly = ax.axvline(color='r')  # the vert line

    def mouse_move(self, event):
        if not event.inaxes:
            return
        self.ly.set_data([event.xdata, event.xdata], [0, 1])
        self.figure.draw()

def runViewer(datapath= None, remoteDir=None, startWithFile=None):
    """ Main function for graphical application start """
    app = QtWidgets.QApplication(sys.argv)
    form = Auger_Window(datapath= datapath, remoteDir=remoteDir, startWithFile=startWithFile)
    form.show()
    sys.exit(app.exec_())

#################  Command line handling  ######################
def validDate(string):
    date = datetime.strptime(string, "%Y-%m-%d")
    return date
def is_valid_file(string):
    if not os.path.exists(string):
        parser.error("The file %s does not exist!" % string)
        return False
    else:
        return string

def myMain():
    parser = argparse.ArgumentParser()
    # parser.add_argument("-v", "--verbosity", type=int, choices=[0, 1, 2],help="increase output verbosity")
    parser.add_argument("-s", "--start", help="Start date (2018-01-01)", default='2019-01-01', type=validDate)
    #parser.add_argument("-e", "--end", help="End date and time (2018-01-01_08:00)", type=validDate, required=True)
    parser.add_argument("-f", "--file", help="Start viewer with this datafile", type=is_valid_file)
    args = parser.parse_args()
    if args.file:
        fPath,fName = os.path.split(args.file)
        # print(">>> ", fPath, " <<< ", fName)
    else:
        fPath = "."
        fName = None

    runViewer(datapath=fPath, startWithFile=fName)
if __name__ == '__main__':
	myMain()


#!/usr/bin/env python3
import numpy as np
import json
import sys, re, os
from libhreels.HREELS import myPath

libDir = os.path.dirname(os.path.realpath(__file__)) 


# Experimental setup as dictionary:
setup = {
    "e0": 4.0,
    "theta": 60.,
    "phia": 2.0,
    "phib": 2.0,
    "temperature": 298.,
    "debug": False
}
# Instrumental function describing elastic peak shape:
instrument = {
    "width": 19.,
    "intensity": 100000.,
    "asym": 0.1,
    "gauss": 0.8
}

	
def importMaterials(string, path=libDir):
    file = os.path.join(myPath(path),'materials.json')
    with open(file) as json_file:
        materials = json.load(json_file)
        try:
            mat = materials[string]
        except:
            print('No data for material >>{}<< found in {} materials.json!!'.format(string, path))
            mat = 'None'
    return mat
	
###########################################################
try:
	from libhreels import myEels2 as LambinEELS   # wrapper for myEels2.f90
	from libhreels import myBoson as LambinBoson  # wrapper for myBoson.f90
except:
	print('myEels2 and MyBoson are not available here (Check your version)')			

class lambin:
    def __init__(self, film, setup=setup, instrument=instrument):
        if not sys.version.startswith('3.6'):
            print('''Make sure the Fortran routines 'myEels2' and 'myBoson' 
            have been complied with the proper f2py3 for the right 
            python version!!''') 
        self.e0 = setup['e0']
        self.theta = setup['theta']
        self.phia = setup['phia']
        self.phib = setup['phib']
        self.temperature = setup['temperature']
        self.debug = setup['debug']
        self.width = instrument['width']
        self.gauss = instrument['gauss']
        self.intensity = instrument['intensity']
        self.asym = instrument['asym']
        self.layers = len(film)          # number of layers
        self.neps = self.layers
        name_size = self.layers
        self.name = []; self.thick=[]; self.listNOsci=[]; self.epsinf =[]; Q = []
        allTO=[]; allgTO=[];  allgLO=[]; nDrude=0
        name2 = []
        for layer in film:
            try:
                a = layer[0]['name']
            except:
                a = 'None'
            self.name.append('{:<10}'.format(a[:10]))        # film name and material
            name2.append(a)
            try:
                a = layer[1]
            except:
                a = 10000.
            self.thick.append(a)
            self.epsinf.append(layer[0]['eps'])
            nTO = 2 * len(layer[0]['wTO'])
            allTO.extend(layer[0]['wTO'])
            allgTO.extend(layer[0]['gTO'])
            allTO.extend(layer[0]['wLO'])
            allgTO.extend(layer[0]['gLO'])
            Q.extend(2*len(layer[0]['wTO'])*[10.])
            self.listNOsci.append(nTO)
        
        if len(allTO)!=sum(self.listNOsci) or len(allgTO)!=sum(self.listNOsci):
            print('Error in materials: ', layer[0])
        self.wOsc = np.array(allTO)
        self.gOsc = np.array(allgTO)
        self.osc = np.array([self.wOsc, np.array(Q), self.gOsc])
        return

    def calcSurfaceLoss(self,x):
        ''' Calculate the surface loss spectrum for the array of x, which needs to be an equidistant array. 
        All parameters are defined in the class __init__() call.'''
        wmin = min(x)
        wmax = max(x)-0.001
        dw = (wmax-wmin)/(len(x)-1)     # assumes that x is an equidistant array
        wn_array_size = len(x)     # size of array for x and epsilon (wn_array, loss_array)
        nper = 1.
        contrl = '{:<10}'.format('None'[:10])   # Can be 'image' to include image charge
        mode = '{:<10}'.format('kurosawa'[:10])           
        wn_array,loss_array = LambinEELS.mod_doeels.doeels(self.e0,self.theta,self.phia,self.phib,
            wmin,wmax,dw,self.layers,self.neps,nper,self.name,
            self.thick,self.epsinf,self.listNOsci,self.osc,contrl,mode,wn_array_size)
        i=0
        for item in wn_array:
            if item > 0: break
            i += 1
        return wn_array[i-1:], loss_array[i-1:]

    def calcHREELS(self,x, normalized=True):
        emin = min(x)
        emax = max(x)-0.001
        norm = 1
        xLoss,loss_array = self.calcSurfaceLoss(x)
        wmin = min(xLoss)
        wmax = max(xLoss)
        xOut,spectrum,n = LambinBoson.doboson3(self.temperature,self.width,self.gauss,self.asym,
            emin,emax,wmin,wmax,loss_array,self.debug,len(loss_array))
        if normalized:
            norm = max(spectrum[:n])
        return xOut[:n], spectrum[:n]/norm

    def calcEps(self, x):
        epsArray = []
        nOsci = len(self.wOsc)
        for wn in x:
            yn = LambinEELS.mod_doeels.seteps(self.listNOsci,nOsci,self.osc,self.epsinf,wn,self.layers)
            epsArray.append(yn)
        return np.transpose(np.array(epsArray))

####################################################################################
def myMain():
    import matplotlib.pyplot as plt
    oxide = importMaterials('NiO')
    substrate = importMaterials('Ag')
    x = np.linspace(-100.,1000.,1000)
    for d in [4., 20., 50., 200., 10000.]:
        film1 = lambin(film=[[oxide,d],[substrate,1000.]])
        # xs, spectrum = film1.calcSurfaceLoss(x)
        xs, spectrum = film1.calcHREELS(x, normalized=True)
        plt.plot(xs,spectrum, label=d)
    plt.ylabel('HREELS Intensity')
    plt.xlabel('Energy Loss (cm$^{-1}$)')
    plt.show()

if __name__ == '__main__':
	myMain()


#!/usr/bin/python
# HREELS library Version 1.0    (October 2018 @ WFW)
#
import numpy as np
import re
# import elog
import os
import matplotlib.pyplot as plt
from scipy import optimize
from scipy.signal import find_peaks_cwt
from datetime import datetime
from datetime import timedelta
from matplotlib.transforms import offset_copy
# from pylab import figure
from matplotlib.pyplot import figure

if os.name == 'nt':
    diskd = 'D:\\'
elif os.name == 'posix':
    diskd = '/mnt/d/'
else:
    print('Define diskd!   System not known ...')

def myPath(path):
    '''Convert any Windows or Linux path to a Linux path. Adds a slash at the end, if missing.'''
    if path == None:
        return path
    try:
        newPath = re.sub(r'\\', '/', path)
        if not newPath[-1] == '/':
            newPath += '/'
        return newPath
    except:
        return path

def offset(ax, x, y):
    return offset_copy(ax.transData, x=x, y=y, units='dots')

def createDataDictionary(fileList, datapath):
    '''A python data dictionary is build from a given list of HREELS data files. All data files are read in and 
    are available with their properties, e.g. d[file].fname, d[file].maxIntensity, d[file].resolution, 
    d[file].xdata, or d[file].ydata.
    '''
    d = {}
    for file in fileList:
        d[file] = HREELS(file, datapath=datapath)
        # print('File:',d[file].fname,' -- ', d[file].maxIntensity, d[file].resolution)
    return d

def HREELS_elog_Dictionary(file, path):
    '''This function reads a HREELS data file (at the given datapath) and creates a dictionary for import into the eLog electronic
    labbook. In addition, the maximal count rate and the elastic peak width are also reported.
    '''
    spec = HREELS(file,datapath=path)
    spec.infoText()
    dic = {'Category':'HREELS', 'Type':'HREELS', 'Substrate':'as before', 'Summary':str(spec.energy)+':%5.0f'% spec.xmin0 +'-%-5.0f'% spec.xmax0, 
        'Temperature':'', 'Pressure_HREELS':'', 'Pressure_Prep':'', 'Pressure_FEL':'', 'Pressure_PreVac':'', 
        'Material':'', 'Energy':str(spec.energy), 'Location':'', 'Duration':str(spec.totalTime), 'File':str(file), 'Thickness':'', 
        'Scan_Range':'%5.0f'% spec.xmin0 +','+'%5.0f'% spec.xmax0, 'Position_x':'', 'Position_y':'', 'Position_z':'', 'Position_phi':'', 
        'Time_Channel':str(spec.timePerChannel), 
        'Filament':'%4.2f'%spec.filament, 'N_channels':spec.channelText, 'Resolution':'%5.1f'%spec.resolution, 'MDate':spec.startTimeEpoch, 
        'MaxCounts':'%7.0f'%spec.maxIntensity, 'Author':'Python Script'}
    return dic


class HREELS:
    """This class handles HREELS data and their plotting. """ 
    path = "\\\\backup-ep3\\BackUp02\\0_experiments\\EELS-PC4\HREELS\\gph_dat\\"
    # path = "./"
    def __init__(self, _filename, datapath=path):
        global eV2cm
        eV2cm = 8065.6
        self.maxIntensity = 1.
        self.datapath = myPath(datapath)
        if self.readDataFile(_filename):
            try:
                self.getElasticPeak()
                self.correctOffset()
                print('File: %s: Counts: %6.0f, Res: %4.1f' % (self.fname, self.maxIntensity, self.resolution))
            except:
                print("Error fitting elastic peak.")
                self.maxIntensity = 1.
                self.resolution = 0
                self.xOffset = -999
            self.valid = True
        else:
            print('No valid datafile:', _filename)
            self.valid = False

    def readDataFile(self, _filename):
        '''Read DELTA 0.5 HREELS data file and extracts xy data in counts/s and cm-1.
        '''
        if _filename.find(".") == -1:
            _filename +=".GPH"
        self.filename = _filename
        self.fname = self.filename[0:self.filename.find(".")]
        xdata = []
        ydata = []
        self.segments = []
        file =os.path.join(self.datapath,self.filename)
        with open(file, 'r', encoding='iso-8859-15') as f:
            f.readline()
            code=f.readline().split()[2]
            if code != "DELTA":
                print ("Data file has wrong format!",code)
                return False
            for line in f:
                if re.search(r"^S \d+ ([-+]?[0-9]*\.?[0-9]+)",line):
                    xdata.append(line.split()[2])
                if re.search(r"^M \d+ (\d+\.\d+)",line):
                    ydata.append(line.split()[2])
                match = re.search('(.*)\t#\tStartzeit',line)
                if match:
                    self.startTime = datetime.strptime(match.group(1), "%m-%d-%Y %H:%M:%S")
                    self.startTimeEpoch = self.startTime.timestamp()
                match = re.search('(.*)\t#\tEndzeit',line)
                if match:
                    self.stopTime = datetime.strptime(match.group(1), "%m-%d-%Y %H:%M:%S")
                match = re.search('(.*)\t#\tMeas time',line)
                if match:
                    self.timePerChannel = float(match.group(1))
                match = re.search('(.*)\t#\tNumber of Segments',line)
                if match:
                    self.numberSegments = int(match.group(1))
                match = re.search('(.*)\t#\tSegmentsize',line)
                if match:
                    a = match.group(1).split()
                    self.segments.append((int(a[0]),float(a[2])*eV2cm,float(a[3])*eV2cm,float(a[5])))
                match = re.search('(.*)\t#\tAmpere',line)
                if match:
                    self.filament = float(match.group(1))
                match = re.search('^E0 + ([-+]?[0-9]*\.?[0-9]+)',line)
                if match:
                    self.energy = float(match.group(1))

        self.xdata = eV2cm * np.array(xdata, dtype=float)
        self.ydata = np.array(ydata, dtype=float)/self.timePerChannel
        self.data = list(zip(self.xdata,self.ydata))
        if len(xdata) < 5:
            print('There are no spectral data in file', self.filename)
            self.xmin = 0.
            self.xmax = 0.
            return False
        else:
            self.xmin = self.xdata[0]
            self.xmax = self.xdata[-1]
        self.xmin0 = self.xmin
        self.xmax0 = self.xmax
        self.totalTime = self.stopTime-self.startTime
        self.marker = []
        return True

    def __str__(self):
        return self.datapath+self.filename

    def info(self):
        print("filename: \t\t",self.filename)
        print("datapath: \t\t",self.datapath)
        print("startTime: \t\t",self.startTime)
        print("totalTime: \t\t",self.totalTime)
        print("xmin: \t\t\t","%6.1f"%self.xmin,"cm-1")
        print("xmax: \t\t\t","%6.1f"%self.xmax,"cm-1")
        print("timePerChannel:\t\t","%4.2f"%self.timePerChannel,"s")
        if len(self.segments) > 1:
            print("segments: \t\t")
            for each in self.segments:
                print("\t\t%5d channels: %6.1f to %6.1f cm-1 in %4.2f s per channel"%each)
        else:
            print("channels: \t\t","%5d"%self.segments[0][0])
        if len(self.marker)>0:
            print("marker: \t\t")
            for each in self.marker:
                print("\t\t%6.1f\t %8.1f"%each)
        print("maxIntensity: \t\t","%8.1f"%self.maxIntensity,"counts/s")
        print("resolution: \t\t","%5.1f"%self.resolution,"cm-1")
        print("xOffset: \t\t","%6.3f"%self.xOffset,"cm-1")
        print('--------')

    def infoText(self):
        channelText =" "
        longText = "filename: \t\t"+self.filename+'\n'
        longText +="datapath: \t\t"+self.datapath+'\n'
        longText +="startTime: \t\t"+str(self.startTime)+'\n'
        longText +="totalTime: \t\t"+str(self.totalTime)+'\n'
        longText +="xmin: \t\t\t"+"%6.1f"%self.xmin+" cm-1"+'\n'
        longText +="xmax: \t\t\t"+"%6.1f"%self.xmax+" cm-1"+'\n'
        longText +="timePerChannel:\t\t"+"%4.2f"%self.timePerChannel+" s"+'\n'
        if len(self.segments) > 1:
            longText +="segments: \t\t"+'\n'
            for each in self.segments:
                longText +="\t\t%5d channels: %6.1f to %6.1f cm-1 in %4.2f s per channel"%each+'\n'
                channelText += "%5d channels: %6.1f to %6.1f cm-1 in %4.2f s per channel"%each+'<br>'
        else:
            longText +="channels: \t\t"+"%5d"%self.segments[0][0]+'\n'
            channelText += "%5d"%self.segments[0][0]
        self.channelText = channelText
        if len(self.marker)>0:
            longText +="marker: \t\t"+'\n'
            for each in self.marker:
                longText +="\t\t%6.1f\t %8.1f"%each+'\n'
        longText +="maxIntensity: \t\t"+"%8.1f"%self.maxIntensity+" counts/s"+'\n'
        longText +="resolution: \t\t"+"%5.1f"%self.resolution+" cm-1"+'\n'
        longText +="xOffset: \t\t\t"+"%6.3f"%self.xOffset+" cm-1"+'\n'
        longText +='--------'
        return longText

    def findIndex(self, lossenergy):
        for i in range(len(self.xdata)):
            if self.xdata[i] > lossenergy: 
                return i
        print('HREELS.findIndex: lossenergy', lossenergy, 'not found in xdata')
        return len(self.xdata)-1

    def plot(self, xmin=None, xmax=None, factor=1, label='x', normalized=False, color="b-",marker=True):
        ''' plot(self, xmin=None, xmax=None, factor=1, label='x', normalized=False, color="b-",marker=True)'''
        self.pFactor = factor
        if xmin:
            nstart = self.findIndex(xmin)
        else:
            nstart = 0
        if xmax:
            nend = self.findIndex(xmax)
        else:
            nend = len(self.xdata)
        if normalized:
            factor /= self.maxIntensity
        if label == "x":
            label = self.filename
        
        plt.plot(self.xdata[nstart:nend], factor*self.ydata[nstart:nend], color, label=label)
        self.factor = factor
        plt.xlabel('Energy Loss (cm$^{-1}$)')
        if normalized:
            plt.ylabel('Normalized Intensity')
        else:
            plt.ylabel('Intensity (s$^{-1}$)')
        plt.legend()
        if marker:
            for (x,y) in self.marker:
                self.setMarker(x,y*factor)
        return

    def pickMarker(self):
        def onclick(event):
            # print('%s click: button=%d, x=%d, y=%d, xdata=%f, ydata=%f' %
            #     ('double' if event.dblclick else 'single', event.button,
            #     event.x, event.y, event.xdata, event.ydata))
            if event.button ==1:
                self.marker.append((event.xdata, event.ydata))
                self.setMarker(event.xdata, event.ydata)
                self.fig.canvas.draw()
            else:
                self.marker = []
                print("All marker deleted!")
                self.fig.canvas.draw()
        self.fig.canvas.mpl_connect('button_press_event', onclick)
        self.ax.set_title('Pick marker with left mouse button ... (right:erase all)')

    def figure(self):
        self.fig=figure()
        self.ax=self.fig.add_subplot(111)

    def show(self):
        plt.show()

    def drawAndWait(self):
        plt.draw()
        plt.pause(0.001)
        value = input("Press [enter] to continue.")
        return value

    def func(self, x, a, width, x0):
            return a * np.exp(-((x-x0)/width*1.6651092)**2)

    def gaussQB(self, x, amp, width, x0, const=0., lin=0., q=0.):
        ''' Function for a Gaussian peak with quadratic background.'''
        return amp * np.exp(-((x-x0)/width*1.6651092)**2) + const + lin *(x-x0) + q*(x-x0)**2

    def getElasticPeak(self):
        if self.xmin < -20.:
            initialWidth = 20.
            xmin_Index =self.findIndex(-initialWidth)
            xmax_Index =self.findIndex(initialWidth)
            # (ftx,fity) is the data range to fit the elastic peak initially
            fitx = self.xdata[xmin_Index:xmax_Index]
            fity = self.ydata[xmin_Index:xmax_Index]
            #print(xmax_Index-xmin_Index)
            if (xmax_Index-xmin_Index < 10):
                return np.array([ 0., 0., 0.])
            popt, dummy = optimize.curve_fit(self.func, fitx, fity, p0=[28000., 8.,0.], bounds=([0., 2., -8.], [900000., 30., 8.]))
            #print('Fit: I=%5.1f kc/s, FWHM=%4.1f cm^-1, Offset=%5.3f cm^-1' % tuple(popt))
            # We now take a smaller x range that is centered around the peak position and fit again:
            xmin_Index2 =self.findIndex(-popt[1]*0.55)
            xmax_Index2 =self.findIndex(popt[1]*0.55)
            fitx = self.xdata[xmin_Index2:xmax_Index2]
            fity = self.ydata[xmin_Index2:xmax_Index2]
            popt, dummy = optimize.curve_fit(self.func, fitx, fity, p0=[28000., 8.,0.], bounds=([0., 2., -8.], [900000., 30., 8.]))
            # print('Fit: I=%5.1f kc/s, FWHM=%4.1f cm^-1, Offset=%5.3f cm^-1' % tuple(popt))
            self.maxIntensity = popt[0]
            self.resolution = popt[1]
            self.xOffset = popt[2]
            return popt
        else:
            print('Dataset without elastic peak.')
            self.maxIntensity = 1.
            self.resolution = 0.
            self.xOffset = 0.


    def getPeak(self, x0, y0):
        initialWidth = 60.
        xmin_Index =self.findIndex(x0-initialWidth)
        xmax_Index =self.findIndex(x0+initialWidth)
        # (ftx,fity) is the data range to fit the elastic peak initially
        fitx = self.xdata[xmin_Index:xmax_Index]
        fity = self.ydata[xmin_Index:xmax_Index]*self.factor
        popt, dummy = optimize.curve_fit(self.func, fitx, fity, p0=[y0, 20.,x0], bounds=([y0/4., 2., x0-50.], [y0*3., 90., x0+50.]))
        # print('Fit: I=%5.1f kc/s, FWHM=%4.1f cm^-1, peak=%5.3f cm^-1' % tuple(popt))
        self.fit = popt
        return popt

    def getPeakQB(self, x0, y0):
        ''' Fits a Gaussian peak centered around x0 with quadratic background.'''
        initialWidth = 80.
        xmin_Index =self.findIndex(x0-initialWidth)
        xmax_Index =self.findIndex(x0+initialWidth)
        fitx = self.xdata[xmin_Index:xmax_Index]
        fity = self.ydata[xmin_Index:xmax_Index]*self.factor
        startValues = [y0,      30.,    x0,     2.,     0., 0.]   # amp, width, x0, const, lin, q
        lowerBounds = [y0/4.,   10.,    x0-60,  0.,    -10000., 0.]  
        upperBounds = [y0*3.,   80.,   x0+60,  1000000.,   10000., 0.001]  
        popt, dummy = optimize.curve_fit(self.gaussQB, fitx, fity, p0=startValues, bounds=(lowerBounds, upperBounds))
        x0 = popt[2]
        xmin_Index =self.findIndex(x0-initialWidth)
        xmax_Index =self.findIndex(x0+initialWidth)
        fitx = self.xdata[xmin_Index:xmax_Index]
        fity = self.ydata[xmin_Index:xmax_Index]*self.factor
        startValues = popt
        popt, dummy = optimize.curve_fit(self.gaussQB, fitx, fity, p0=startValues, bounds=(lowerBounds, upperBounds))
        print('Fit: I=%5.1f kc/s, FWHM=%4.1f cm^-1, peak=%5.3f cm^-1, const=%5.3f, lin=%5.3f, q=%6.5f' % tuple(popt))
        self.fit = popt
        x = np.linspace(self.fit[2]-initialWidth,self.fit[2]+initialWidth)
        self.fitCurve = self.gaussQB(x, self.fit[0],self.fit[1],self.fit[2],self.fit[3],self.fit[4],self.fit[5])
        return popt

    def getPeakFromMarker(self, index):
        # popt = self.getPeak(self.marker[index][0],self.marker[index][1])
        popt = self.getPeak(self.marker[index][0],self.marker[index][1])
        self.marker[index] =(popt[2],popt[0]/self.factor)
        # print('Fit peak:',popt[2],popt[0]/self.factor)

    def showPeak(self):
        initialWidth = 80.
        x0 = self.fit[2]
        x = np.linspace(x0-initialWidth,x0+initialWidth)
        if len(self.fit)==3:
            fit = self.func(x, self.fit[0],self.fit[1],self.fit[2])
        elif len(self.fit)==6:
            fit = self.gaussQB(x, self.fit[0],self.fit[1],self.fit[2],self.fit[3],self.fit[4],self.fit[5])
        else:
            print('Wrong number of fit arguments!')
            return
        plt.plot(x,fit)
        return

    def selectData(self,x1, x2=None):
        """Returns the HREELS data as (x,y) list between the
        wavenumbers x1 and x2. If x2 is omitted the full range is used."""
        if not x2:
            x2 = self.xmax
        a = self.findIndex(x1)
        b = self.findIndex(x2)+1
        return self.data[a:b]

    def plotInfoAmp(self,factor=400,xmin=300,marker=True):
        self.info()
        self.figure()
        self.plot(label=self.fname+"\n"+str(self.startTime),marker=False)
        self.plot(xmin=xmin, factor=factor, color='r-', label='x%i' % factor, marker=marker)
        #self.drawAndWait()

    def labelFactor(self, xlabel):
        nxlabel = self.findIndex(xlabel)
        plt.text(self.xdata[nxlabel],self.ydata[nxlabel]*self.factor*1.1,'x%3i' % self.pFactor,  
        verticalalignment='bottom', horizontalalignment='left')

    def pickPeak(self):
        def onclick(event):
            if event.button ==1:
                self.getPeakQB(event.xdata,event.ydata)
                self.marker.append((self.fit[2],self.fit[0]/self.factor))
                self.setMarker(self.fit[2],self.fit[0])
                self.showPeak()
                self.fig.canvas.draw()
            else:
                self.marker.pop()
                print("Marker #{} deleted!".format(len(self.marker)+1))
                self.fig.canvas.draw()
        self.fig.canvas.mpl_connect('button_press_event', onclick)
        self.ax.set_title('pick Marker with left button ... (right button:Erase last)')

    def setMarker(self, x, y, ymin=0, size=None):
        ''' Set vertical marker with text label. Note that self.figure() needs to be 
        called before use.'''
        trans = offset(self.ax, 0, 30)
        plt.plot([x,x],[ymin,y], lw=1, c='black', ls='dashed')
        x = round(x)
        plt.text(x,y,'%4i' % x, rotation=90, 
        verticalalignment='bottom', horizontalalignment='center', transform=trans)
        return

    def correctOffset(self):
        self.xdata -= self.xOffset
        self.data = list(zip(self.xdata,self.ydata))
        self.xmin = self.xdata[0]
        self.xmax = self.xdata[-1]
        # self.xOffset = 0.

    def peakFind(self,xStart=60, marker=False):
        ''' Finds all peaks and returns a list of their x values. If marker='True', 
        markers will be added to an existing plot.'''
        x2 = self.xdata[self.findIndex(xStart):]
        y2 = self.ydata[self.findIndex(xStart):]
        peakind = find_peaks_cwt(y2, np.arange(11,28))
        for i in peakind[1:]:
            print('Detected peak at %5.1f' % x2[i])
        if (marker):
            for i in peakind[1:]:
                self.setMarker(x2[i],y2[i])
        return self.xdata[self.findIndex(xStart):][peakind]

    def plotWaterFall(self, fileList, offset, xmin=0, xmax=0, colorSeq=False, reverse=False, factor=1., legend=False, normalized=False, myColor=None):
        if xmin:
            nstart = self.findIndex(xmin)
        else:
            nstart = 0
        if xmax:
            nend = self.findIndex(xmax)
        else:
            nend = len(self.xdata)
        label = None
        i = 0.
        iMax = float(len(fileList))
        if normalized:
            normFactor = factor/self.maxIntensity
        if reverse:
            fileList = reversed(fileList)
        for item in fileList:
            self.__init__(item, datapath=self.datapath)
            if normalized:
                normFactor = factor/self.maxIntensity
            else:
                normFactor = factor
            if myColor:
                plotColor = myColor
            else:
                plotColor=plt.cm.RdYlBu(i/iMax)
            plt.plot(self.xdata[nstart:nend], normFactor*self.ydata[nstart:nend]+i*offset, color=plotColor, label=self.fname)
            i += 1.
        if legend:
            plt.legend()
        return
        

def myMain():
    fig = None
    datapath = diskd+ "/Data/Python/HREELS/expHREELS/data"
    data1 = HREELS("I2L02",datapath)
    if data1.valid:
        data1.info()
        data1.figure()
        data1.plot(label=data1.fname+"\n"+str(data1.startTime))
        data1.plot(xmin=300, factor= 10, color='r-', label='')
        if data1.drawAndWait() == 'm':
            data1.plot(xmin=1400, factor= 600, color='g.', label='')

    a = data1.infoText()
    print('InfoText:\n',a)
    print('Filament: {:4.2f} A'.format(data1.filament))
    print('Energy: {:2.0f} eV'.format(data1.energy))



if __name__ == '__main__':
	myMain()


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['libhreels']

package_data = \
{'': ['*']}

install_requires = \
['PyQt5>=5.13,<6.0',
 'argparse>=1.4.0,<2.0.0',
 'matplotlib>=3.2,<4.0',
 'numpy>=1.18.2,<2.0.0',
 'requests>=2.23.0,<3.0.0',
 'scipy>=1.4,<2.0']

entry_points = \
{'console_scripts': ['calchreels = libhreels.calcHREELS:myMain',
                     'dielectrics = libhreels.dielectrics:myMain',
                     'viewhreelstest = libhreels.ViewHREELS:myMain'],
 'gui_scripts': ['viewauger = libhreels.ViewAuger:myMain',
                 'viewhreels = libhreels.ViewHREELS:myMain']}

setup_kwargs = {
    'name': 'libhreels',
    'version': '1.0.6',
    'description': 'Handling, simulating, and plotting HREELS and Auger spectroscopy data',
    'long_description': '# HREELS data handling and simulation\n\nSet of data analysis routines for surface vibrational spectroscopy based on a Delta05 spectrometer. Simple data reading and plotting routines are provided, but also advanced routines for in depth analysis.\n\nA fast data browser as graphical user interface is included as **ViewHREELS.py**. It can also take command line arguments.\n\nAdditional a python interface to a Fortran-based calculation of full HREEL spectra is provided in **class lambin** (in **calcHREELS.py**). Note that this part requires a local compilation (via f2py3) from the Fortran90 files. Only for the Linux-based python version 3.6, there are complied files provided. For details see below.\n\nMost of the routines are within the **class HREELS** (HREELS.py). A simple command line program is "showHREELS.py", which reads one data file and plots the spectrum with a second amplified trace.\n\n    Usage:  >python showHREELS.py filename [factor wavenumber]\n\n        E.g.: python showHREELS.py H9H03 100 53 \n\nMore general usage:\n\nRead dataset by calling the HREELS class:\n    **data1 = HREELS(\'filename\', datapath=\'datapath\')**        # Here you can omit the extension \'.gph\'\n                                                    # The second argument is optional\n                                                    \n    This will assign/calculate the following properties:\n    \n    data1.filename\n    data1.datapath\n    data1.startTime\n    data1.stopTime\n    data1.totalTime\n    data1.timePerChannel\n    data1.numberOfSegments\n    data1.energy    # Electron kinetic energy\n    data1.filament  # Filament current in Ampere\n    data1.segments  # list of segment info\n    data1.data      # [(-100.1021616, 259.5), (-98.5374352, 264.5), ...]\n    data1.xdata\n    data1.ydata\n    data1.maxIntensity  # Count rate of elastic peak\n    data1.resolution    # width of elastic peak\n\nThe following methods are defined within the class:\n\n    info()      :   Print information about the spectrum\n    plot()      :   Draws the spectrum. Optional arguments are:\n                    (xmin=None, xmax=None, factor=1, label=\'x\', normalized=False, color="b-",marker=True)\n\n    plotInfoAmp()   Draws spectrum together with amplified trace.\n    pickPeak()  :   Select peak by mouse cursor. The call of figure() is required before.\n    selectData():   Returns the data between wavenumbers x1 and x2\n    findIndex(lossenergy): Returns the data array index for a given energy loss\n    setMarker(x, y, ymin=0, size=None): Sets vertical marker with text label \n    plotWaterFall(...):\n        \n\n# calcHREELS\n\nThese routines are based on the publication "Computation of the surface electron-energy-loss spectrum in specular geometry for an arbitrary plane-stratified medium" by P. Lambin, J.-P. Vigneron, and A. A. Lucas, in the Journal "Computer Physics Communications 60, 351-64(1990)".\n\n\n\n',
    'author': 'Wolf Widdra',
    'author_email': 'wolf.widdra@physik.uni-halle.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitlab.informatik.uni-halle.de/e3fm8/libhreels',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
